package controllers;

import java.io.File;
import java.io.FileInputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import models.GeopapProject;
import models.GpsImage;
import models.GpsLog;
import models.GpsNote;
import play.mvc.Controller;
import play.mvc.Http.MultipartFormData;
import play.mvc.Http.MultipartFormData.FilePart;
import play.mvc.Http.RawBuffer;
import play.mvc.Http.RequestBody;
import play.mvc.Result;
import eu.hydrologis.utils.CompressionUtilities;
import eu.hydrologis.utils.FileUtilities;
import eu.hydrologis.utils.KmzUtilities;
import eu.hydrologis.utils.LineChartGenerator;
import eu.hydrologis.utils.Utilities;

public class Application extends Controller {

    public static Result index() {

        String filter = "*";

        RequestBody body = request().body();
        Map<String, String[]> queryData = request().queryString();
        if (queryData != null) {
            String[] filterData = queryData.get("filter");
            if (filterData != null && filterData.length > 0) {
                filter = filterData[0];
            }
        }

        return redirect(routes.Application.projects(filter));
    }

    public static Result projects( String filter ) {
        List<GeopapProject> all = GeopapProject.all();

        if (filter != null && filter.length() != 0 && !filter.equals("*")) {
            List<GeopapProject> filtered = new ArrayList<GeopapProject>();
            for( GeopapProject geopapProject : all ) {
                if (Utilities.matches(geopapProject, filter)) {
                    filtered.add(geopapProject);
                }
            }
            all = filtered;
        }
        return ok(views.html.index.render(all, filter));
    }

    public static Result uploadFile() {
        return ok(views.html.loadfile.render());
    }

    public static Result upload() {
        String userName = "UNKNOWN";
        String description = "";

        boolean fromForm = false;
        /*
         * check the upload type
         */
        File file = null;
        RawBuffer asRaw = request().body().asRaw();
        if (asRaw != null) {
            file = asRaw.asFile();

            Map<String, String[]> headers = request().headers();
            Set<Entry<String, String[]>> entrySet = headers.entrySet();
            for( Entry<String, String[]> entry : entrySet ) {
                String key = entry.getKey();
                if (key.trim().toUpperCase().equals("AUTHORIZATION")) {
                    String[] value = entry.getValue();
                    if (value[0].contains(":")) {
                        String[] split = value[0].split(":");
                        userName = split[0];
                    }
                }
            }
        } else {
            RequestBody body = request().body();

            MultipartFormData multiFormData = body.asMultipartFormData();
            Map<String, String[]> formData = multiFormData.asFormUrlEncoded();
            if (formData != null) {
                String[] usernameData = formData.get("username");
                if (usernameData != null && usernameData.length > 0) {
                    userName = usernameData[0];
                }
                String[] descriptionData = formData.get("description");
                if (descriptionData != null && descriptionData.length > 0) {
                    description = descriptionData[0];
                }
            }

            FilePart geopaparazziZip = multiFormData.getFile("geopaparazzizip");
            file = geopaparazziZip.getFile();
            fromForm = true;
        }

        String message = "Error in uploading the file";
        try {
            SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            File allProjectsFolder = Utilities.getAllProjectsFolder();
            File userFolder = Utilities.getUserFolder(userName, allProjectsFolder);

            String projectName = CompressionUtilities.unzipFolder(file.getAbsolutePath(), userFolder.getAbsolutePath(), true);
            System.out.println(userFolder + "/" + projectName);
            File projectFolder = new File(userFolder, projectName);

            File descriptionFile = Utilities.getDescriptionFile(projectFolder);
            if (descriptionFile.exists()) {
                description = description + FileUtilities.readFile(descriptionFile);
            }

            GeopapProject p = new GeopapProject();
            p.userName = userName;
            p.name = projectName;
            p.description = description;
            p.date = dateFormatter.format(new Date());
            p.folderPath = userName + "/" + projectName;
            GeopapProject.create(p);

            // now create the lines kml
            List<GpsLog> logs = Utilities.getLogs(projectFolder);
            File kmlFile = new File(projectFolder, "lines.kml");
            KmzUtilities.createKml(kmlFile, logs);

            message = "Properly uploaded project: " + projectName;
        } catch (Exception e) {
            e.printStackTrace();
        }

        if (fromForm) {
            return ok(views.html.index.render(GeopapProject.all(), "*"));
        } else {
            return ok(message);
        }
    }

    public static Result delete( Long id ) {
        GeopapProject geopapProject = GeopapProject.byId(id);
        File projectFolder = Utilities.getProjectFolderByName(geopapProject.userName, geopapProject.name);
        FileUtilities.deleteFileOrDir(projectFolder);
        GeopapProject.delete(id);

        return redirect(routes.Application.projects("*"));
    }

    public static Result show( Long id ) throws Exception {

        GeopapProject geopapProject = GeopapProject.byId(id);
        File projectFolder = Utilities.getProjectFolderByName(geopapProject.userName, geopapProject.name);
        List<GpsImage> images = Utilities.getImages(projectFolder);
        List<GpsNote> notes = Utilities.getNotes(projectFolder);
        List<GpsLog> logs = Utilities.getLogsDescriptions(projectFolder);

        return ok(views.html.viewproject.render(geopapProject, images, notes, logs));
    }

    public static Result download() throws Exception {
        StringBuilder sb = new StringBuilder();
        try {
            List<GeopapProject> geopapProjects = GeopapProject.all();
            sb.append("{").append("\n");
            sb.append("  \"projects\":[").append("\n");
            for( int i = 0; i < geopapProjects.size(); i++ ) {
                GeopapProject p = geopapProjects.get(i);
                if (i > 0) {
                    sb.append("     ,").append("\n");
                }
                sb.append("     {").append("\n");
                sb.append("     \"id\":\"").append(p.id).append("\",\n");
                sb.append("     \"title\":\"").append(p.name).append("\",\n");
                sb.append("     \"date\":\"").append(p.date).append("\",\n");
                sb.append("     \"author\":\"").append(p.userName).append("\",\n");
                sb.append("     \"name\":\"").append(p.description).append("\",\n");
                sb.append("     \"size\":\"").append("").append("\"\n");
                sb.append("     }").append("\n");
            }
            sb.append("  ]").append("\n");
            sb.append("}").append("\n");
        } catch (Exception e) {
            sb.append("{").append("\n");
            sb.append("  An error occurred!").append("\n");
            sb.append("}").append("\n");
            e.printStackTrace();
        }

        return ok(sb.toString());
    }

    public static Result downloadProject( Long id ) throws Exception {
        GeopapProject p = GeopapProject.byId(id);
        File projectFolder = Utilities.getProjectFolderByName(p.userName, p.name);
        File zipFile = new File(projectFolder.getParentFile(), projectFolder.getName() + ".zip");
        if (zipFile.exists()) {
            zipFile.delete();
        }
        CompressionUtilities.zipFolder(projectFolder.getAbsolutePath(), zipFile.getAbsolutePath(), true);
        FileInputStream inputStream = null;
        try {
            inputStream = new FileInputStream(zipFile);
            zipFile.delete();
            return ok(inputStream).as("zip/stream");
        } catch (Exception e) {
            e.printStackTrace();
            return ok("An error occurred during the download of the project.");
        }
    }

    public static Result logChart( Long projectId, Long logId ) throws Exception {
        GeopapProject p = GeopapProject.byId(projectId);
        File projectFolder = Utilities.getProjectFolderByName(p.userName, p.name);
        GpsLog gpsLog = Utilities.getLogById(projectFolder, logId);

        LineChartGenerator gen = new LineChartGenerator("Profile chart for: " + gpsLog.text, "Distance [degrees]", "Elevation");
        List<GpsNote> points = gpsLog.points;
        GpsNote previous = null;
        double[][] data = new double[points.size()][2];
        double distance = 0;
        for( int i = 0; i < points.size(); i++ ) {
            GpsNote current = points.get(i);
            if (i == 0) {
                data[i][0] = distance;
                data[i][1] = Double.parseDouble(current.altim);
            } else {
                double pLat = Double.parseDouble(previous.lat);
                double pLon = Double.parseDouble(previous.lon);
                double cLat = Double.parseDouble(current.lat);
                double cLon = Double.parseDouble(current.lon);
                double dx = Math.abs(cLon - pLon);
                double dy = Math.abs(cLat - pLat);

                distance = distance + Utilities.pythagoras(dx, dy);
                data[i][0] = distance;
                data[i][1] = Double.parseDouble(current.altim);
            }
            previous = current;
        }
        // for( double[] ds : data ) {
        // System.out.println(ds[0] + "," + ds[1]);
        // }
        gen.addDatasetMatrix(data, "");
        File chartsFolder = Utilities.getChartsFolder();
        File tempFile = File.createTempFile("geopap_browser", "chart.png", chartsFolder);
        gen.dumpChart(tempFile, true, false, 800, 600);

        FileInputStream inputStream = null;
        try {
            inputStream = new FileInputStream(tempFile);
            tempFile.delete();
            return ok(inputStream).as("image/png");
        } catch (Exception e) {
            e.printStackTrace();
            return ok("An error occurred during the creation of the chart image.");
        }
    }
}