package controllers;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;

import models.GeopapProject;
import models.GpsLog;
import models.GpsNote;

import org.geotools.data.simple.SimpleFeatureCollection;
import org.geotools.feature.FeatureCollections;
import org.geotools.feature.simple.SimpleFeatureBuilder;
import org.geotools.feature.simple.SimpleFeatureTypeBuilder;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.jgrasstools.gears.io.vectorwriter.VectorWriter;
import org.jgrasstools.gears.utils.geometry.GeometryUtilities;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.simple.SimpleFeatureType;

import play.mvc.Controller;
import play.mvc.Result;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.Point;

import eu.hydrologis.utils.CompressionUtilities;
import eu.hydrologis.utils.FileUtilities;
import eu.hydrologis.utils.Utilities;

public class Gis extends Controller {

    public static Result notesToShape( Long projectId ) throws Exception {
        GeopapProject geopapProject = GeopapProject.byId(projectId);
        File projectFolder = Utilities.getProjectFolderByName(geopapProject.userName, geopapProject.name);
        List<GpsNote> notes = Utilities.getNotes(projectFolder);

        SimpleFeatureTypeBuilder b = new SimpleFeatureTypeBuilder();
        b.setName("geopaparazzinotes"); //$NON-NLS-1$
        b.setCRS(DefaultGeographicCRS.WGS84);
        b.add("the_geom", Point.class); //$NON-NLS-1$
        b.add("DESCRIPTION", String.class);
        b.add("TIMESTAMP", String.class);
        b.add("ALTIM", String.class);

        SimpleFeatureType featureType = b.buildFeatureType();
        SimpleFeatureCollection newCollection = FeatureCollections.newCollection();

        for( GpsNote gpsNote : notes ) {
            String dateTimeString = gpsNote.dateTime;
            String text = gpsNote.text;

            // and then create the features
            double lat = Double.parseDouble(gpsNote.lat);
            double lon = Double.parseDouble(gpsNote.lon);
            Coordinate c = new Coordinate(lon, lat);
            Point point = GeometryUtilities.gf().createPoint(c);

            SimpleFeatureBuilder builder = new SimpleFeatureBuilder(featureType);
            Object[] values = new Object[]{point, text, dateTimeString, gpsNote.altim};
            builder.addAll(values);
            SimpleFeature feature = builder.buildFeature(null);
            newCollection.add(feature);
        }

        File chartsFolder = Utilities.getChartsFolder();
        File tmp = File.createTempFile("geopap-browser", "download", chartsFolder);
        String name = tmp.getName();
        tmp.delete();
        File folderFile = new File(chartsFolder, name);
        folderFile.mkdir();
        File tmpShapefile = File.createTempFile("geopap-browser", "notes.shp", folderFile);
        VectorWriter.writeVector(tmpShapefile.getAbsolutePath(), newCollection);

        File newFile = new File(chartsFolder, name + ".zip");

        CompressionUtilities.zipFolder(folderFile.getAbsolutePath(), newFile.getAbsolutePath(), true);

        FileUtilities.deleteFileOrDir(folderFile);

        FileInputStream inputStream = null;
        try {
            inputStream = new FileInputStream(newFile);
            newFile.delete();
            return ok(inputStream).as("zip/stream");
        } catch (Exception e) {
            e.printStackTrace();
            return ok("An error occurred during the creation of the notes shapefile.");
        }
    }

    public static Result logsToShape( Long projectId ) throws Exception {
        GeopapProject geopapProject = GeopapProject.byId(projectId);
        File projectFolder = Utilities.getProjectFolderByName(geopapProject.userName, geopapProject.name);
        List<GpsLog> logs = Utilities.getLogs(projectFolder);

        SimpleFeatureTypeBuilder b = new SimpleFeatureTypeBuilder();
        b.setName("geopaparazzilogs");
        b.setCRS(DefaultGeographicCRS.WGS84);
        b.add("the_geom", MultiLineString.class);
        b.add("STARTDATE", String.class);
        b.add("ENDDATE", String.class);
        b.add("DESCR", String.class);

        SimpleFeatureType featureType = b.buildFeatureType();
        SimpleFeatureCollection newCollection = FeatureCollections.newCollection();

        GeometryFactory gf = GeometryUtilities.gf();
        for( GpsLog log : logs ) {
            List<GpsNote> points = log.points;

            List<Coordinate> coordList = new ArrayList<Coordinate>();
            String startDate = log.startTime;
            String endDate = log.endTime;
            for( GpsNote gpsPoint : points ) {
                double lat = Double.parseDouble(gpsPoint.lat);
                double lon = Double.parseDouble(gpsPoint.lon);
                Coordinate c = new Coordinate(lon, lat);
                coordList.add(c);
            }
            Coordinate[] coordArray = (Coordinate[]) coordList.toArray(new Coordinate[coordList.size()]);
            if (coordArray.length < 2) {
                continue;
            }
            LineString lineString = gf.createLineString(coordArray);
            MultiLineString multiLineString = gf.createMultiLineString(new LineString[]{lineString});

            SimpleFeatureBuilder builder = new SimpleFeatureBuilder(featureType);
            Object[] values = new Object[]{multiLineString, startDate, endDate, log.text};
            builder.addAll(values);
            SimpleFeature feature = builder.buildFeature(null);

            newCollection.add(feature);
        }

        File chartsFolder = Utilities.getChartsFolder();
        File tmp = File.createTempFile("geopap-browser", "download", chartsFolder);
        String name = tmp.getName();
        tmp.delete();
        File folderFile = new File(chartsFolder, name);
        folderFile.mkdir();
        File tmpShapefile = File.createTempFile("geopap-browser", "logs.shp", folderFile);
        VectorWriter.writeVector(tmpShapefile.getAbsolutePath(), newCollection);

        File newFile = new File(chartsFolder, name + ".zip");

        CompressionUtilities.zipFolder(folderFile.getAbsolutePath(), newFile.getAbsolutePath(), true);

        FileUtilities.deleteFileOrDir(folderFile);

        FileInputStream inputStream = null;
        try {
            inputStream = new FileInputStream(newFile);
            newFile.delete();
            return ok(inputStream).as("zip/stream");
        } catch (Exception e) {
            e.printStackTrace();
            return ok("An error occurred during the creation of the log shapefile.");
        }
    }

}