/*
 * Geopaparazzi - Digital field mapping on Android based devices
 * Copyright (C) 2010  HydroloGIS (www.hydrologis.com)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package eu.hydrologis.utils;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;

import models.GpsLog;

/**
 * Kml utils
 * 
 * @author Andrea Antonello (www.hydrologis.com)
 */
@SuppressWarnings("nls")
public class KmzUtilities {


    public static void createKml(File outputFile, List<GpsLog> gpsLogs) throws IOException {
          String  name = "Geopaparazzi GpsLog";

        /*
         * write the kml file
         */
        BufferedWriter bW = null;
        try {
            bW = new BufferedWriter(new FileWriter(outputFile));
            bW.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
            bW.write("<kml xmlns=\"http://www.opengis.net/kml/2.2\" xmlns:gx=\"http://www.google.com/kml/ext/2.2\"\n");
            bW.write("xmlns:kml=\"http://www.opengis.net/kml/2.2\" xmlns:atom=\"http://www.w3.org/2005/Atom\">\n");
            bW.write("<Document>\n");
            bW.write("<name>");
            bW.write(name);
            bW.write("</name>\n");
            bW.write("<Style id=\"red-pushpin\">\n");
            bW.write("	<IconStyle>\n");
            bW.write("		<scale>1.1</scale>\n");
            bW.write("		<Icon>\n");
            bW.write("			<href>http://maps.google.com/mapfiles/kml/pushpin/red-pushpin.png\n");
            bW.write("			</href>\n");
            bW.write("		</Icon>\n");
            bW.write("		<hotSpot x=\"20\" y=\"2\" xunits=\"pixels\" yunits=\"pixels\" />\n");
            bW.write("	</IconStyle>\n");
            bW.write("	<ListStyle>\n");
            bW.write("	</ListStyle>\n");
            bW.write("</Style>\n");
            bW.write("<Style id=\"yellow-pushpin\">\n");
            bW.write("	<IconStyle>\n");
            bW.write("		<scale>1.1</scale>\n");
            bW.write("		<Icon>\n");
            bW.write("			<href>http://maps.google.com/mapfiles/kml/pushpin/ylw-pushpin.png\n");
            bW.write("			</href>\n");
            bW.write("		</Icon>\n");
            bW.write("		<hotSpot x=\"20\" y=\"2\" xunits=\"pixels\" yunits=\"pixels\" />\n");
            bW.write("	</IconStyle>\n");
            bW.write("	<ListStyle>\n");
            bW.write("	</ListStyle>\n");
            bW.write("</Style>\n");
            bW.write("	<StyleMap id=\"blackline\">\n");
            bW.write("		<Pair>\n");
            bW.write("			<key>normal</key>\n");
            bW.write("			<styleUrl>#blackline0</styleUrl>\n");
            bW.write("		</Pair>\n");
            bW.write("	</StyleMap>\n");
            bW.write("	<Style id=\"blackline0\">\n");
            bW.write("		<LineStyle>\n");
            bW.write("			<color>000000</color>\n");
            bW.write("			<width>3</width>\n");
            bW.write("		</LineStyle>\n");
            bW.write("	</Style>\n");

            for( GpsLog log : gpsLogs ) {
                try {
                    bW.write(log.toKmlString());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            bW.write("</Document>\n");
            bW.write("</kml>\n");

        } finally {
            if (bW != null)
                bW.close();
        }
    }
}
